"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var identifierRegex = /^[a-zA-Z_$][a-zA-Z_$0-9]*$/;

/*
  If the part is a function, we need to get the actual value of the parameters.
  A parameter x or y in func(x, y) may be
    a) Assigned a value in one of the dicts
    b) OR should be treated as a string literal "x" or "y"
*/
function getArgumentValue(dicts) {
  return function (a) {
    return a === "true" || a === "false" ? { value: a === "true" } : isNaN(a) ? identifierRegex.test(a) ? function () {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = dicts[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var dict = _step.value;

          if (typeof dict === "function") {
            var val = dict(a);
            if (val) {
              return val;
            }
          } else {
            if (Object.prototype.hasOwnProperty.call(dict, a)) {
              return { value: dict[a] };
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return { value: a };
    }() : { error: a + " is not a valid identifier." } : { value: +a };
  };
}

/*
    Split path expression into
      a) objects
      b) functions and their parameters
*/
function analyzePath(encodedPath, dicts) {
  //The path would be url encoded
  var path = decodeURI(encodedPath);
  var parts = path.indexOf(".") ? path.split(".") : [path];
  return parts.map(function (part) {
    var openingBracket = part.indexOf("(");
    var isFunction = openingBracket > -1;
    return isFunction ? function () {
      var closingBracket = part.indexOf(")");
      var identifier = part.substring(0, openingBracket);
      var argsString = part.substring(openingBracket + 1, closingBracket);
      var args = argsString.split(",").map(function (a) {
        return a.trim();
      }).map(getArgumentValue(dicts));
      return { type: "function", identifier: identifier, args: args };
    }() : { type: "object", identifier: part };
  });
}

var RoutingError = exports.RoutingError = function RoutingError(message, type) {
  _classCallCheck(this, RoutingError);

  this.message = message;
  this.type = type;
};

function isIterable(gen) {
  return gen[Symbol.iterator] && typeof gen[Symbol.iterator] === "function" || Symbol.asyncIterator && gen[Symbol.asyncIterator] && typeof gen[Symbol.asyncIterator] === "function";
}

function __isIterable(gen) {
  return gen.next && typeof gen.next === "function";
}

exports.default = function () {
  var _ref = _asyncToGenerator(function* (_app, expression) {
    var dicts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

    var iterateToEnd = function () {
      var _ref2 = _asyncToGenerator(function* (resultOrGenerator) {
        if (__isIterable(resultOrGenerator)) {
          var gen = resultOrGenerator;
          while (true) {
            var nextVal = yield gen.next();
            if (options.onNextValue && !nextVal.done) {
              options.onNextValue((yield nextVal.value));
            }
            if (nextVal.done) {
              return yield nextVal.value;
            }
          }
        } else {
          return resultOrGenerator;
        }
      });

      return function iterateToEnd(_x5) {
        return _ref2.apply(this, arguments);
      };
    }();

    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};


    var app = typeof _app === "function" ? _app() : _app;
    var additionalArgs = options.args || [];
    var parts = expression ? analyzePath(expression, dicts) : [];

    var obj = void 0,
        error = void 0,
        current = app;

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = parts[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var part = _step2.value;

        obj = obj ? obj + "." + part.identifier : "" + part.identifier;
        if (typeof current !== "undefined") {
          if (options.beforeAccess) {
            current = options.beforeAccess(current, part.identifier);
          }
          if (part.type === "function") {
            var fn = current[part.identifier];
            if (typeof fn === "function") {
              var resultOrGenerator = yield fn.apply(current, options.prependArgs ? additionalArgs.concat(part.args.map(function (a) {
                return a.value;
              })) : part.args.map(function (a) {
                return a.value;
              }).concat(additionalArgs));
              current = yield iterateToEnd(resultOrGenerator);
            } else if (typeof fn === "undefined") {
              error = new RoutingError("The requested path was not found.", "NOT_FOUND");
              break;
            } else {
              error = new RoutingError(obj + " is not a function. Was " + (typeof fn === "undefined" ? "undefined" : _typeof(fn)) + ".", "NOT_A_FUNCTION");
              break;
            }
          } else {
            var ref = current[part.identifier];
            var _resultOrGenerator = yield typeof ref === "function" ? ref.apply(current, additionalArgs) : ref;
            current = yield iterateToEnd(_resultOrGenerator);
          }
        } else {
          break;
        }
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    var finalResult = error ? error : (typeof current === "undefined" ? "undefined" : _typeof(current)) === "object" && current.hasOwnProperty(options.index) ? yield _asyncToGenerator(function* () {
      if (options.beforeAccess) {
        current = options.beforeAccess(current, options.index);
      }
      var resultOrGenerator = typeof current[options.index] === "function" ? current[options.index].apply(current, additionalArgs) : current[options.index];
      return yield iterateToEnd(resultOrGenerator);
    })() : current;

    return yield finalResult;
  });

  function route(_x, _x2) {
    return _ref.apply(this, arguments);
  }

  return route;
}();
//# sourceMappingURL=nsoap.js.map